package sudoku;

import org.junit.jupiter.api.RepeatedTest;
import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.*;
import java.util.stream.IntStream;

class SolverTest {
    @Test
    void constructorTest() {
        Solver solver = new Solver();
        assertNotNull(solver);
    }

    @Test
    void setBoardTest() {
        Solver solver = new Solver();
        int[][] board = new int[9][9];
        solver.setBoard(board);
        assertEquals(board, solver.getBoard());
    }

    @Test
    void randomizeBoardTest() {
        int[][] board = new int[9][9];
        Solver solver = new Solver();
        solver.randomizeBoard();
        assertNotEquals(board, solver.getBoard());
    }

    @Test
    void legalTest() {
        Solver solver = new Solver();
        assertTrue(solver.isLegal(0, 0, 1));
        solver.set(0, 0, 1);

        IntStream.range(0, 9).forEach(i -> {
            assertFalse(solver.isLegal(0, i, 1));
            assertFalse(solver.isLegal(i, 0, 1));
        });

        assertTrue(solver.isLegal(5, 5, 1));
        assertTrue(solver.isLegal(8, 8, 9));
        assertTrue(solver.isLegal(8, 8, 1));
    }

    @Test
    @RepeatedTest(100)
    void solverTest() {
        Solver solver = new Solver();
        assertFalse(solver.isSolved());
        assertTrue(solver.solve());
        assertTrue(solver.isSolved());

        solver.clear();
        assertFalse(solver.isSolved());
        assertTrue(solver.solve());
    }

    @Test
    @RepeatedTest(100)
    void randomizeBoardGuaranteeSolvableTest() {
        Solver solver = new Solver();
        solver.randomizeBoard();
        assertTrue(solver.solve());
    }

    @Test
    void resetTest() {
        Solver solver = new Solver();
        assertTrue(solver.solve());
        solver.randomizeBoard();
        solver.clear();
        assertFalse(solver.isSolved());
    }

    @Test
    void legalWithInvalidInputsTest() {
        Solver solver = new Solver();
        assertFalse(solver.isLegal(-1, 0, 1));
        assertFalse(solver.isLegal(0, -1, 1));
        assertFalse(solver.isLegal(0, 0, -1));
        assertFalse(solver.isLegal(0, 0, 10));
    }

    @Test
    void unsolvableTest() {
        Solver solver = new Solver();
        
        // Simple example
        solver.clear();
        solver.set(0, 0, 1);
        solver.set(0, 1, 1);
        assertFalse(solver.solve());

        // More complex example
        solver.clear();
        solver.set(0, 5, 7);
        solver.set(0, 6, 8);
        solver.set(0, 7, 2);
        solver.set(1, 5, 3);
        solver.set(2, 7, 1);
        solver.set(3, 1, 4);
        solver.set(5, 2, 8);
        solver.set(5, 8, 6);
        solver.set(7, 1, 8);
        assertFalse(solver.solve());
    }

    @Test
    void setBoardInvalidInputThrowsTest() {
        Solver solver = new Solver();
        assertThrows(NullPointerException.class, () -> solver.setBoard(null));
        assertThrows(IllegalArgumentException.class, () -> solver.setBoard(new int[8][8]));
    }
}
